##################################################################
#
# Library routines to hash a text file into word counts.
#
# $Id: FourNineTeen.pm,v 1.11 2005/03/16 15:44:56 devel Exp $
#
##################################################################
#
#    Copyright 2005 Charles Shapiro 
#    This file is part of the Fraudulator.
#
#    The Fraudulator is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    The Fraudulator is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with the Fraudulator; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#############################################################################

package FourNineTeen;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(makeHash makeHashBuf read_hashfile printHashDebug 
makeHashVector findMessageInList okToPrint printHashList);

use strict;

# 
# Make a hash list of the words in a given buffer.
# Entry Point
#
sub makeHashBuf($$)
{
    my ($inputBuf,$wordsNumRef) = @_;
    my %outHash;
    my $inputW;
    
    foreach $inputW (split /\W+/,$inputBuf)
    {
	++$$wordsNumRef;
	if( ($inputW !~ /[^a-zA-Z0-9]/) && ($inputW)) 
	{
	    $inputW =~ tr/[A-Z]/[a-z]/;
	    if( $outHash{$inputW} )
	    {
		$outHash{$inputW}++;
	    }
	    else
	    {
		$outHash{$inputW}=1;
	    }
	}
    }
    %outHash;
}
#
# Make a hash list of the words in given file.
# Entry Point.
# 
sub makeHash($$)
{
    my ( $filename,$wordsNumRef ) = @_;
    my $numWords;
    my %wordshash;
    open(INTEXT,"<$filename");
    my @wordsBuf = <INTEXT>;
    %wordshash = makeHashBuf(join("",@wordsBuf),\$numWords);
    $$wordsNumRef = $numWords;
    %wordshash;
}


#
# Read in average letter vector file.
#
sub read_hashfile($)
{
    my $avgVecFilename=$_[0];
    my $inLine;
    my $inTag;
    my $inNum;
    my %retHash;
    my $success=0;
    my $sequenceNum=1;

    if(! open(AVGVEC,"<$avgVecFilename"))
    {
	$success=1;
    }
    if(0 == $success)
    {
	while($inLine=<AVGVEC>)
	{
	    ($inNum,$inTag)=split(/\s+/,$inLine);
	    $retHash{$inTag}=$sequenceNum;
	    ++$sequenceNum;
	}
    }
    %retHash;
}

#
# Print a raw letter vector (debug)
# Entry Point
#
sub printHashDebug(\%)
{
    my $hashK;
    my %printObj=%{$_[0]};
    my $hashLen;
    my @hashKeys;

    @hashKeys=sort(keys(%printObj));

    foreach $hashK (@hashKeys)
    {
	print($printObj{$hashK},"   ",$hashK,"\n");
    }
    $hashLen=$#hashKeys;
    print("Length: ",$hashLen,"\n");
    print("\n");
}

#
# Make a hash of an individual letter by analyzing its words
# in relation to the global list of words.
# Entry Point
#
sub makeHashVector($$$)
{
    my ( $globalHashRef,$letterWordHashRef,$letterLen ) = @_;
    my $hashVecKey;
    my @outList;
    my $retVal;
    my $scaleFactor=10000;
    my $scaled_present;

    if($letterLen)
    {
	$scaled_present = int($scaleFactor / $letterLen);
    }
    else
    {
	$scaled_present=1;
    }
    foreach $hashVecKey (sort keys %{$globalHashRef} )
    {
    	if ( $$letterWordHashRef{$hashVecKey} )
        {	    
	    push(@outList,$scaled_present);
        }
	else
	{
	    push(@outList,"0");
	}
    }
#    my $retVal=pack('u*',@outList);
    my $retVal=join("-",@outList);
    $retVal;
}

#
# Given a pair of vectors, compute the angle between them.
#
sub compute_theta($$)
{
    my $ii;
    my $retVal;
    my @vecOneList=split(/-/,$_[0]);
    my @vecTwoList=split(/-/,$_[1]);
    my $arrayLen=scalar(@vecOneList);

    while( $ii < $arrayLen)
    {
	$retVal += $vecOneList[$ii] * $vecTwoList[$ii];
        ++$ii;
    }

#    print($retVal, "<br>\n");
    return $retVal;
}
#
# Sort a list of files and theta values by theta value.
#
sub byTheta()
{
    my $firstVal;
    my $twostVal;
    my $dummy;

    ($firstVal,$dummy)=split(/--/,$a);
    ($twostVal,$dummy)=split(/--/,$b);

    return $twostVal <=> $firstVal;
}
#
# Given a message vector and a list of message vectors, return
# the vectors with the largest theta (that is, the vectors 
# of the letters which most resemble the message vector).
# Entry Point
#
sub findMessageInList($$$$$)
{
    my($fileName,$letterKey,$listSize,$exactMatchNumRef,$identityScoreRef) = @_;
    my $fileKey;
    my $inputLine;
    my $lettertheta;
    my $identitytheta;
    my $thetaKey;
    my @retVal;
    
    $identitytheta=compute_theta($letterKey,$letterKey);
    $$identityScoreRef=$identitytheta;
    open(INTEXT,"<$fileName");
    while ($inputLine=<INTEXT>) 
    {
	($fileKey,$fileName)=split(/\s+/,$inputLine);
	$lettertheta=compute_theta($fileKey,$letterKey);
	if($lettertheta == $identitytheta)
	{
	    $$exactMatchNumRef++;
	}
	push(@retVal,$lettertheta . "--" . $fileName);
    }
    close(INTEXT);
    @retVal = sort byTheta @retVal;
    @retVal=splice(@retVal,0,$listSize);
    @retVal;
}

#
# Does this fall within the correct range?
#
sub okToPrint($$$$)
{
    my $hashWord=$_[0];
    my $numTimes=$_[1];
    my $status = 1;
    my $upper = $_[2];
    my $lower = $_[3];
    if($upper > -1)
    {
	if($numTimes > $upper)
	{
	    $status=0;
	}
    }
    if( ($numTimes < $lower) && ($status) )
    {
	$status=0;
    }
   
    return $status;
}

#
# Sort a word hash by number.
#
sub byNum()
{
    my $onestNum=$a;
    my $twostNum=$b;
    
    $onestNum =~ s/\s.*//;
    $twostNum =~ s/\s.*//;
    $onestNum <=> $twostNum;
#    $twostNum <=> $onestNum;
}

#
# Print the hash out when done.
#
sub printHashList($$$%)
{
#   print(STDERR "printHash(%) ...\n");
    my $fileName=shift(@_);
    my $upperB=shift(@_);
    my $lowerB=shift(@_);
    my %hashToPrint=@_;
    my $hashK;
    my @output;
    my $outputCounter;
    my $outputString;

    foreach $hashK (keys %hashToPrint)
    {
	if (okToPrint($hashK,$hashToPrint{$hashK},$upperB,$lowerB))
	{
	    $outputString = sprintf("%d\t%s",$hashToPrint{$hashK},$hashK);
	    $output[$outputCounter] = $outputString;
	    ++$outputCounter;
	}
    }
#   print(STDERR "Sorting.. \n");
    @output = sort byNum @output;
    open(OUTPUTFILE,">$fileName");
    while(@output)
    {
	print(OUTPUTFILE $output[0],"\n");
	shift(@output);
    }
    print(OUTPUTFILE "\n");
    close(OUTPUTFILE);
}









