#!/usr/bin/perl
###############################################################################
#
# Create a hash vector by words. Given a global word hash and a letter file,
# produce a number consisting of the position in the global hash of each 
# word in the letter corresponding to a word in the global hash.
#
# $Id: hashFinderLib.pl,v 1.6 2005/03/16 15:45:24 devel Exp $
#
###############################################################################
#
#    Copyright 2005 Charles Shapiro 
#    This file is part of the Fraudulator.
#
#    The Fraudulator is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    The Fraudulator is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with the Fraudulator; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#############################################################################


# use lib("../419Lib");
use strict;
use Getopt::Std;

use FourNineTeen;

use vars qw($opt_h $opt_i $opt_L $opt_s);

#
# Usage Message
#
sub usage($)
{
    print("Usage: $_[0] -h hashfile -i infile ");
    print("-L hashlistfile -s size\n\n");
    print<<EOU;
        Find the list of files in hashlistfile similar to infile by hashfile.
        Get a list of size files, with the most similar file at the top.
EOU

print('$Id: hashFinderLib.pl,v 1.6 2005/03/16 15:45:24 devel Exp $',"\n");
}

#
# Main Line
#

if (! getopts("i:h:L:s:"))
{
    usage($0);
    die;
}
if (! $opt_i)
{
    usage($0);
    die("No -i infile\n");
}
if(! $opt_h)
{
    usage($0);
    die("No -h hashfile\n");
}
if(! $opt_L)
{
    usage($0);
    die("No -L listfile\n");
}

my %allHash = read_hashfile($opt_h);
# printHash(%allHash);

if(! %allHash)
{
    die("Cannot read hashfile $opt_h.");
}

if (! $opt_s)
{
    $opt_s=20;
}

my $letterSize;
my %letterHash=makeHash($opt_i,\$letterSize);
my $hashVecKey = makeHashVector(\%allHash,\%letterHash,$letterSize);
my $exactMatchNum;
my $identityScore;
my @fileList = findMessageInList($opt_L,$hashVecKey,$opt_s,
\$exactMatchNum,\$identityScore);
####################################################################
my $listMemb;
print("Identity: ",$identityScore,"\n");
print("Number of identical files: ",$exactMatchNum,"\n");
my $score;
my $filename;
print("File\tScore\n");
foreach $listMemb (@fileList)
{
    ($score,$filename)=split(/--/,$listMemb);
    print($filename,"\t",$score,"\n");
}













